#include "date.h"


/* Notes :
Pour s'assurer de la validité des attributs d'une classe, on doit mettre des contrôles dans les sélecteurs de type set...
Exemple : si un attribut _z doit être compris entre x et y, on fait ce test dans la méthode setZ de sorte qu'à  tout moment on s'assure que _z est bien entre x et y.
De plus, à  chaque fois que l'on doit modifier _z, on passera par setZ qui fera ce test (y compris dans le constructeur par initialisation).
Ici, la situation est un peu différente car la plage des valeurs autorisées pour _jour dépend de la valeur de _mois (et de _annee pour le mois de février). On ne peut donc pas faire (ou difficilement) ce type de contrôle.
On passera alors par une méthode (estValide) qui retourne vrai si la date est valide : elle teste la validité de l'ensemble des attributs de la classe plutôt que de faire un contrôle pour la modification de chaque attribut
En résumé, dans la classe Date, les contrôles de validité ne se feront pas dans les méthodes 'set...' mais via 'estValide'.
*/


//-----------------------------
// Constructeurs/destructeurs
//-----------------------------

Date::Date() {
  _jour  = 1;
  _mois  = 1;
  _annee = 0;
}

Date::Date(int iJour, int iMois, int iAnnee) {
  _jour  = iJour;
  _mois  = iMois;
  _annee = iAnnee;
  if (!estValide()){
    // on remet la date par défaut
    _jour  = 1;
    _mois  = 1;
    _annee = 0;
  }
}

Date::Date(const Date & iDate){
  // on peut mettre à  jour les attributs car on considère que d est valide
  _jour = iDate.getJour();
  _mois = iDate.getMois();
  _annee = iDate.getAnnee();
}

Date::~Date() {
}


//-----------------------------
// Selecteurs
//-----------------------------

int Date::getJour() const {
  return _jour;
}

int Date::getMois() const {
  return _mois;
}

int Date::getAnnee() const {
  return _annee;
}

void Date::setJour(int iJour) {
  int oldValue = _jour;
  _jour = iJour;
  if (!estValide()){
    // on remet l'ancienne valeur
    cout<<"avertissement : attribut _jour non modifié"<<endl;
    _jour = oldValue;
  }
}

void Date::setMois(int iMois) {
  int oldValue = _mois;
  _mois = iMois;
  if (!estValide()){
    // on remet l'ancienne valeur
    cout<<"avertissement : attribut _mois non modifie"<<endl;
    _mois = oldValue;
  }
}

void Date::setAnnee(int iAnnee){
  int oldValue = _annee;
  _annee = iAnnee;
  if (!estValide()){ // c'est le cas si on avait par exemple le 29/2/2008 et que l'on modifie l'année en 2007
    // on remet l'ancienne valeur
    cout<<"avertissement : attribut _mois non modifie"<<endl;
    _annee = oldValue;
  }
}


//-----------------------------
// Autres Methodes
//-----------------------------


// input
//-------
void Date::input(istream & flux) {
    // on demande la saisie de la date tant que celle-ci n'est pas valide
    do {
        if (flux==cin){
          cout << "Saisie de la date";
          cout << endl << "Jour : ";
        }
        flux >> _jour;
        if (flux==cin){
          cout << endl << "Mois : ";
        }
        flux >> _mois;
        if (flux==cin){
          cout << endl << "Annee : ";
        }
        flux >> _annee;
        //cout << endl;
    }
    while (! estValide());
}


// affiche
//--------
void Date::output(ostream & flux) const {
    flux << _jour << "/" << _mois << "/" << _annee << endl;
}

// estValide
//----------
bool Date::estValide() const {
    return ( (_mois >= 0) && (_mois <= 12) && (_jour >= 1) && (_jour <= nbJoursDuMois(_mois, _annee)) );
}



//-----------------------------
// Surcharges d'operateurs
//-----------------------------

Date & Date::operator = (const Date & d){
  // d est une Date valide, on se contente donc de recopier les attributs
  setJour(d.getJour());
  setMois(d.getMois());
  setAnnee(d.getAnnee());
  return (*this);
}


bool Date::operator < (const Date& d) const {
  bool res = true;
  if (_annee > d.getAnnee()){
    res = false;
  }
  else{
    if (_annee == d.getAnnee()){
      if (_mois > d.getMois()){
        res = false;
      }
      else{
        if (_mois == d.getMois()){
          if (_jour > d.getJour()){
            res = false;
          }
        }
      }
    }
  }
  return res;
}

bool Date::operator > (const Date& d) const {
  bool res = true;
  if (_annee < d.getAnnee()){
    res = false;
  }
  else{
    if (_annee == d.getAnnee()){
      if (_mois < d.getMois()){
        res = false;
      }
      else{
        if (_mois == d.getMois()){
          if (_jour < d.getJour()){
            res = false;
          }
        }
      }
    }
  }
  return res;
}

bool Date::operator == (const Date& d) const {
  bool res = false;
  if ((_annee == d.getAnnee()) && (_mois == d.getMois()) && (_jour == d.getJour())){
    res = true;
  }
  return res;
}


bool Date::operator != (const Date& d) const {
  bool res = ! ((*this)==d);
  return res;
}

bool Date::operator <= (const Date& d) const {
  bool res = ( ((*this)<d) || ((*this)==d) );
  return res;
}

bool Date::operator >= (const Date& d) const {
  bool res = ( ((*this)>d) || ((*this)==d) );
  return res;
}



// opérateurs de flux
//-------------------
istream& operator >> (istream& fi, Date& d) {
  d.input(fi);
  return fi;
}


ostream& operator << (ostream& fo, const Date& d) {
  d.output(fo);
  return fo;
}




// opérateurs de flux
//-------------------
ifstream& operator >> (ifstream& fi, Date& d) {
    fi >> d._jour;
    fi >> d._mois;
    fi >> d._annee;

    if(! d.estValide()){
        d.setJour(1);
        d.setMois(1);
        d.setAnnee(0);
    };

    return fi;
}


ofstream& operator << (ofstream& fo, const Date& d) {
    fo << d._jour << "/"<< d._mois << "/" << d._annee;
    return fo;
}






//-----------------
// Autres fonctions
//-----------------

bool isAnneeBissextile(int iAnnee) {
    // une annee est bissextile si elle est divisible par 4 et par par 100, a mois
    // qu'elle soit divisible par 400
    return ( (iAnnee%4 == 0) && ((! (iAnnee%100 == 0)) || (iAnnee%400 == 0)) );
}

int nbJoursDuMois(int iMois, int iAnnee) {
    int res=0;
    switch(iMois) {
    case 1:
    case 3:
    case 5:
    case 7:
    case 8:
    case 10:
    case 12:
        res = 31;
        break;
    case 4:
    case 6:
    case 9:
    case 11:
        res = 30;
        break;
    case 2: if (isAnneeBissextile(iAnnee)){
                res = 29;
            }
            else{
              res = 28;
            };
            break;
    default:
        // cas ou le mois n'est pas valide : message d'erreur
        cout << "Erreur: " << iMois << " n'est pas un mois valide!" << endl;
        res = -1;
    }
    return res;
}

